(* $Id: godi_pax.ml 678 2007-02-25 17:16:54Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2005 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

(* TODO: Call pax and gzip by configurable paths *)

open Printf

let default_protocol =
  `Forward (fun s -> prerr_string s; flush stderr)
;;


let create_archive ?interactivity ?(protocol = default_protocol)
                   ~in_files ~out_file () =
  (* We create a temporary directory with symlinks to the roots of [in_files].
   * These symlinks are later discarded from the recorded names using
   * "-s" transformations.
   *)
  let tmpdir = Godi_file.allocate_tmpdir() in
  let tmp = Godi_file.tmpdir_path tmpdir in
  
  let n = ref 0 in
  List.iter
    (fun (root,_) ->
       if root = "" || root.[0] <> '/' then
	 failwith "Godi_pax.create_archive: roots must be absolute paths";
       Godi_file.create_symlink 
	 root
	 (Filename.concat tmp ("root" ^ string_of_int !n));
       incr n
    )
    in_files;

  (* Write a file with links to the input files: *)

  let (_, list) =
    List.fold_right
      (fun (root, files) (n, list) ->
	 let prefix = "root" ^ string_of_int n in
	 let files' = 
	   List.map
	     (fun file ->
		if file = "" || file.[0] = '/' then
		  failwith "Godi_pax.create_archive: files must be relative paths";
		Filename.concat prefix file
	     )
	     files in
	 (n-1, files' @ list)
      )
      in_files
      ((List.length in_files - 1), []) in

  let list_file = Filename.concat tmp "list" in
  Godi_file.write_file_lines list_file list;

  let success_file = Filename.concat tmp "success" in
  let cmd =
    sprintf
      "{ cd %s && godi_pax -w -x ustar -s ':^root[0-9]*/::' < %s && touch %s; } | gzip -9 -c > %s"
      (Godi_file.uquote tmp)
      (Godi_file.uquote list_file)
      (Godi_file.uquote success_file)
      (Godi_file.uquote out_file) in

  let sys_cmd = Godi_exec.shell cmd  in

  try
    Godi_exec.call_cmd ~stderr:protocol ?interactivity sys_cmd;
    if not (Godi_file.file_exists success_file) then
      raise (Godi_exec.Command_failure(fst sys_cmd, snd sys_cmd, 
				       Godi_exec.WEXITED 1));
    Godi_file.release_tmpdir tmpdir
  with
    | error ->
	(try Godi_file.remove out_file with _ -> ());
	Godi_file.release_tmpdir tmpdir;
	raise error
;;
